/*
 *     JrbxMainFrame.java  Jan 21, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.gui;
import org.jtool.platform.gui.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

public class JrbxMainFrame extends MainFrame {
    private Refactor refactor;
    private RefactoringMenuPane refMenuPane;
    private RefactoringMenuPane refPopupPane;
    private HistoryAllDialog historyAllDialog;

    public JrbxMainFrame(SimpleBrowser browser, String appName) {
        super(browser, appName);

        refactor = new Refactor(this);
    }

    protected MenuPane createMenuPane() {
        MenuPane pane = super.createMenuPane();

        JMenu refMenu = new JMenu("Refcatoring");
        refMenu.setMnemonic('R');
        pane.addMenu(refMenu, 2);

        refMenuPane = new RefactoringMenuPane(this, refactor, refMenu);
        refMenu.addMenuListener(refMenuPane);

        historyAllDialog = new HistoryAllDialog(this);
        RefactoringHistory.setHistoryAllDialog(historyAllDialog);
        RefactoringHistory.init();
        pane.addOptionsMenuItem(new JSeparator());
        pane.addOptionsMenuItem(createAllHistoryMenu(this));

        AboutDialog dialog = new AboutDialog(this);
        pane.setAboutDialog(dialog);

        return pane;
    }

    private JMenuItem createAllHistoryMenu(final JFrame frame) {
        JMenuItem historyAllItem = new JMenuItem("Show History All...");
        historyAllItem.setMnemonic('H');
        historyAllItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                CodePane codePane = getCodesPane().getCurrentCodePane();

                if (!historyAllDialog.isVisible()) {
                    historyAllDialog.update();

                    Point point = frame.getLocationOnScreen();
                    historyAllDialog.setLocation(point.x + frame.getSize().width / 2, point.y + 10);

                    historyAllDialog.setVisible(true);
                }
            }
        });

        return historyAllItem;
    }

    protected CodesPane createCodesPane() {
        CodesPane pane = super.createCodesPane();

        JPopupMenu refPopup = pane.getJPopupMenu();
        refPopup.add(new JSeparator());

        refPopupPane = new RefactoringMenuPane(this, refactor, refPopup);
        refPopup.addPopupMenuListener(refPopupPane);
        
        return pane;
    }

    protected CodeUndoManager createCodeUndoManager() {
        return new RefactoringUndoManager(this);
    }

    protected LogPane createLogPane() {
        LogPane pane = super.createLogPane();
        refactor.addEventListener(pane);
        
        return pane;
    }

    protected void setAdditionalSettings() {
        refMenuPane.setCodesPane(getCodesPane());
        refPopupPane.setCodesPane(getCodesPane());
        refactor.setCodesPane(getCodesPane());
    }
}
