/*
 *     RefactoringRecord.java  Jan 21, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.gui;
import javax.swing.undo.CompoundEdit;
import java.util.ArrayList;
import java.io.Serializable;
import java.sql.Timestamp;

public class RefactoringRecord implements Serializable {
    private String command;
    private String fileName;
    private Timestamp timestamp;
    private String userName;
    transient private boolean inProgress = false;
    transient private ArrayList changedFiles;  // String
    transient private CompoundEdit edit;
    
    private RefactoringRecord() {
    }

    public RefactoringRecord(String cmd, String fname, ArrayList files) {
        command = new String(cmd);
        fileName = new String(fname);
        changedFiles = new ArrayList(files);
        timestamp = new Timestamp(System.currentTimeMillis());
        userName = new String(System.getProperty("user.name"));

        inProgress = true;
    }

    public String getCommand() {
        return command;
    }

    public String getCommandWithoutID() {
        String prefix = command.substring(0, command.indexOf("#"));
        String suffix = command.substring(command.indexOf(":"));
        return prefix + suffix;
    }

    public String getFileName() {
        return fileName;
    }

    public Timestamp getTimestamp() {
        return timestamp;
    }

    public String getUseName() {
        return userName;
    }

    public boolean isInProgress() {
        return inProgress;
    }

    public ArrayList getChangedFiles() {
        return new ArrayList(changedFiles);
    }

    public void setEdit(CompoundEdit edit) {
        this.edit = edit;
    }

    public CompoundEdit getEdit() {
        return edit;
    }

    public String getLog() {
        return command + "  " + fileName + " by " + userName + " at " + timestamp.toString();
    }

    public String getSimpleLog() {
        return command + " at " + timestamp.toString();
    }
}
