/*
 *     MoveClass.java  Feb 7, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.classes;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.jtool.platform.j2xml.SystemProperties;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

public class MoveClass extends ClassRefactoring {
    private JavaFile dfile;
    private MoveClassTransformer transformer;

    public MoveClass() {
        super();
    }

    public void setUpTest(JFrame f, JavaFile dfile) {
        frame = f;
        this.dfile = dfile;
    }

    protected void preconditions() throws RefactoringException {
        if (dfile == null) {
            dfile = MoveClassDialog.show(frame, "Move Class: " + jclass.getName(), jfile.getBaseName(),
                                         SystemProperties.getProjectDir(), QueryProject.getAllJavaFiles());
        }
        if (dfile == null) {
            throw new RefactoringException("");
        }

        if (QueryFile.hasTheClass(dfile, jclass)) {
            throw new RefactoringException("Not move:" +
              " the class/interface " + jclass.getName() + " already exists in the target file " + getRelativeName(dfile.getName()));
        }

        ArrayList files = QueryProject.getAllJavaFilesUsingTheClass(jclass);
        if (dfile.getPackageName().length() == 0) {
            Iterator it = files.iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();

                if (jf.getPackageName().length() != 0) {
                    throw new RefactoringException("Not move:" +
                      " the class/interface " + jclass.getName() + " can not move to the anonymous package");
                }
            }
        }

        if (!dfile.getPackageName().equals(jfile.getPackageName())) {
            if (jclass.isPublic()) {

                if (frame != null) {
                    int answer = JOptionPane.showConfirmDialog(frame, "Rename Class:" +
                      " You must rename the file name " + jfile.getBaseName() + " to " +
                      dfile.getName() + ".java after moving class.",
                      "Confirm", JOptionPane.YES_NO_OPTION);
                    if (answer == JOptionPane.NO_OPTION) {
                        throw new RefactoringException("");
                    }
                }

            } else {
                Iterator it = files.iterator();
                while (it.hasNext()) {
                    JavaFile jf = (JavaFile)it.next();

                    if (!jf.getPackageName().equals(dfile.getPackageName())) {
                        throw new RefactoringException("Not move:" +
                          " the class/interface " + jclass.getName() + " can not move to a different package");
                    }
                }
            }
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new MoveClassTransformer(jclass);
        transformer.changeSrc();
        unparse(jfile);
        
        transformer.changeDst(dfile);
        unparse(dfile);
    }
    
    protected void additionalTransformation() throws RefactoringException {
        if (dfile.getPackageName().length() != 0) {
            Iterator it = QueryProject.getAllJavaFilesUsingTheClass(jclass).iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();

                transformer.changeRef(jf, dfile.getPackageName());
                unparse(jf);
            }
        }
    }

    protected String getLog() {
        return jclass.getName() + " from " + getRelativeName(jfile.getName()) + " to " +
            getRelativeName(dfile.getName());
    }
}
