/*
 *     RenameClass.java  Feb 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.classes;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import java.io.*;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

public class RenameClass extends ClassRefactoring {
    private String newName = null;
    private RenameClassTransformer transformer;

    public RenameClass() {
        super();
    }

    public void setUpTest(JFrame f, String newName) {
        frame = f;
        this.newName = newName;
    }

    protected void preconditions() throws RefactoringException {
        if (newName == null) {
            newName = RenameDialog.show(frame, "Rename Class", jclass.getName());
        }
        if (newName == null) {
            throw new RefactoringException("");
        }

        if (QueryFile.hasTheClass(jfile, newName)) {
            throw new RefactoringException("Not rename:" +
              " the class/interface " + newName + " already exists in the file " + getRelativeName(jfile.getName()));
        }

        if (QueryFile.usesTheType(jfile, newName)) {
            throw new RefactoringException("Not rename:" + 
              " type " + newName + " is already used in the file " + getRelativeName(jfile.getName()));
        }

        JavaClass jc = QueryFile.getJavaClassWithTheSameName(jfile, newName);
        if (jc != null) {
            throw new RefactoringException("Not rename:" +
              " class/interface " + newName + " already exists in the file " + getRelativeName(jc.getJavaFile().getName()));
        }

        if (jclass.isPublic()) {
            File file = new File(jfile.getDirName(), newName + ".java");
            if (file.exists()) {
                throw new RefactoringException("Not rename:" + 
                  " the file " + file.getName() + " already exists in this project");
            }

            if (frame != null) {
                int answer = JOptionPane.showConfirmDialog(frame, "Rename Class:" +
                  " You must rename the file name " + jfile.getBaseName() + " to " + newName + ".java after renaming class.",
                  "Confirm", JOptionPane.YES_NO_OPTION);
                if (answer == JOptionPane.NO_OPTION) {
                    throw new RefactoringException("");
                }
            }
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new RenameClassTransformer(jclass, newName);
        transformer.changeSrc();
        unparse(jfile);
    }

    protected void additionalTransformation() throws RefactoringException {
        Iterator it = QueryProject.getAllJavaFilesUsingTheClass(jclass).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            transformer.changeRef(jf);
            unparse(jf);
        }
    }

    protected String getLog() {
        return jclass.getName() + " to " + newName + " in " + getRelativeName(jfile.getName());
    }
}
