/*
 *     MoveClassDialog.java  Mar 15, 2004
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.util.Options;
import org.jtool.jrbx.refactor.query.QueryProject;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class MoveClassDialog extends SimpleDialog {
    private JTextField srcNameField;
    private JTextField dstNameField;
    private JavaFile jfile;

    public MoveClassDialog(JFrame frame, String title, String srcName, String dir, ArrayList list) {
        super(frame, title, "Input targe file name:", dir, list);
        srcNameField.setText(srcName);
        setVisible(true);
    }

    protected JPanel createCenterPane(final String dir, final ArrayList list) {
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));

        srcNameField = createInputTextField(box, "Source name: ", 60);
        srcNameField.setBackground(new Color(220, 220, 220));
        srcNameField.setEditable(false);

        JButton selButton = new JButton("File");
        selButton.setBackground(Color.white);
        selButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jfile = FileTreeDialog.show(frame, "File Selection", dir, list);
                dstNameField.setText(jfile.getName());
            }
        });

        dstNameField = createInputTextField(box, "Target name: ", selButton);
        dstNameField.setBackground(Color.white);
        dstNameField.setEditable(false);

        selButton.requestFocus();
        return box;
    }

    protected void okAction() {
        if (jfile != null) {
            setVisible(false);
            dispose();
        }
    }

    protected void cancelAction() {
        jfile = null;
        setVisible(false);
        dispose();
    }

    public static JavaFile show(JFrame frame, String title, String srcName, String dir, ArrayList list) {
        MoveClassDialog dialog = new MoveClassDialog(frame, title, srcName, dir, list);
        return dialog.getJavaFile();
    }

    public JavaFile getJavaFile() {
        return jfile;
    }

    public static void main(String args[]) {
        if (args.length == 3) {
            String dirName = SystemProperties.getCanonicalPath(args[0]);

            SystemProperties.setApplicationDir(".");
            SystemProperties.setDefaultProjectSetting(dirName);
            Options.parse(args);
            SystemProperties.setSapidDir(Options.getOption("-sapiddir"));

            Repository.init();
            Repository.parse();
            Repository.terminate();

            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            JavaFile jfile = MoveClassDialog.show(mainFrame, "Move", "Source.java",
                                                  dirName, QueryProject.getAllJavaFiles());
            if (jfile != null) {
                System.out.println("NAME = " + jfile.getName());
            }
            System.exit(0);
        }
        System.out.println("java MoveClassDialog top_directory -sapiddir sapid_dir");
    }
}
