/*
 *     PreviewDialog.java  Mar 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.jrbx.refactor.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class PreviewDialog extends JDialog {
    private JFrame frame;
    private Container contentPane;
    private JButton okButton;
    private JButton cancelButton;
    private boolean okFlag = false;

    public PreviewDialog(JFrame frame, String title, ArrayList files) {
        super(frame, title, true);
        this.frame = frame;
        createPanes("Confirm changes made to the following files:", files);
        setVisible(true);
    }

    public static boolean show(JFrame frame, String title, ArrayList files) {
        PreviewDialog dialog = new PreviewDialog(frame, title, files);
        return dialog.isOk();
    }

    public boolean isOk() {
        return okFlag;
    }

    private void createPanes(String mesg, ArrayList files) {
        contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());

        JPanel panel = new JPanel();
        JLabel titleLabel = new JLabel(mesg, JLabel.CENTER);
        titleLabel.setForeground(Color.black);
        panel.add(titleLabel);
        contentPane.add(panel, "North");

        ChangeTextsPane changeTextPane = new ChangeTextsPane(files);
        contentPane.add(changeTextPane, "Center");

        panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        createOkButton(panel);
        createCancelButton(panel);
        contentPane.add(panel, "South");

        okButton.requestFocus();

        Point p = frame.getLocationOnScreen();
        this.setLocation(p.x + 100, p.y + 100);
        this.pack();
        this.validate();
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                okFlag = false;
                setVisible(false);
                dispose();
            }
        });
    }

    private void createOkButton(JPanel panel) {
        okButton = new JButton("Ok");
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okFlag = true;
                setVisible(false);
                dispose();
            }
        });
    }

    protected void createCancelButton(JPanel panel) {
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okFlag = false;
                setVisible(false);
                dispose();
            }
        });

        panel.add(okButton);
        panel.add(cancelButton);
    }

    public static void main(String args[]) {
        JFrame mainFrame = new JFrame();
        mainFrame.setSize(100, 100);
        mainFrame.setVisible(true);

        ArrayList al = new ArrayList();
        al.add(new DisplayedCode("/org/jtool/jrbx/refactor/name1", "TEXT 1", "NEW TEXT 1"));
        al.add(new DisplayedCode("/org/jtool/jrbx/name2", "TEXT 2\nTEXT 2", "NEW TEXT 2"));
        al.add(new DisplayedCode("/org/jtool/name3", "TEXT 3\nTEXT 3\nTEXT 3", "NEW TEXT 3"));

        boolean confirm = PreviewDialog.show(mainFrame, "Confirm Changes", al);
        System.out.println("Ok = " + confirm);
        System.exit(0);
    }
}

class ChangeTextsPane extends JTabbedPane implements ChangeListener {
    private ArrayList files;  // DisplayedCode

    public ChangeTextsPane(ArrayList files) {
        super(JTabbedPane.TOP);
        this.files = files;
        init();

        this.addChangeListener(this);
    }

    public void init() {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            DisplayedCode code = (DisplayedCode)it.next();

            JPanel panel = new JPanel();
            panel.setLayout(new BorderLayout());

            JSplitPane textPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
            textPane.setContinuousLayout(true);
            textPane.setDividerSize(2);
            panel.add(textPane, "Center");

            JTextArea oldCodePane = createCodePane(code.getOldCode());
            setHighlights(oldCodePane, code.getOldHighlight());
            textPane.setLeftComponent(createScrollTextPane(oldCodePane, "Before"));

            JTextArea newCodePane = createCodePane(code.getNewCode());
            setHighlights(newCodePane, code.getNewHighlight());
            textPane.setRightComponent(createScrollTextPane(newCodePane, "After"));

            JTextField filenamePane = new JTextField();
            filenamePane.setText("  " + code.getName());
            filenamePane.setEditable(false);
            filenamePane.setBorder(BorderFactory.createEmptyBorder());
            filenamePane.setFont(new Font("Dialog", Font.PLAIN, 14));
            filenamePane.setOpaque(false);
            panel.add(filenamePane, "South");

            this.addTab(code.getBaseName(), panel);
        }
    }

    private JTextArea createCodePane(String code) {
        JTextArea textArea = new JTextArea(28, 70);
        textArea.setText(code);
        textArea.setEditable(false);
        textArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
        textArea.setSelectionColor(Color.gray);
        textArea.setBackground(Color.white);
        textArea.setTabSize(4);
        textArea.setLineWrap(true);
        textArea.setMargin(new Insets(5, 5, 5, 5));
        textArea.setCaretPosition(0);

        return textArea;
    }

    private JScrollPane createScrollTextPane(JTextArea textArea, String title) {
        JScrollPane scTextArea = new JScrollPane(textArea);
        scTextArea.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scTextArea.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        TitledBorder border = new TitledBorder(title);
        border.setTitlePosition(TitledBorder.ABOVE_TOP);
        border.setTitleColor(Color.red);
        scTextArea.setBorder(border);

        return scTextArea;
    }

    private void setHighlights(JTextArea textArea, ArrayList highlights) {
        Highlighter h = textArea.getHighlighter();
        DefaultHighlighter.DefaultHighlightPainter p;

        Iterator it = highlights.iterator();
        while (it.hasNext()) {
            HighlightToken ht = (HighlightToken)it.next();
            p = new DefaultHighlighter.DefaultHighlightPainter(ht.getColor());

            try {
                h.addHighlight(ht.getBegin(), ht.getEnd(), p);
            } catch (BadLocationException e) { }
        }
    }

    public void stateChanged(ChangeEvent evt) {
        JTabbedPane src = (JTabbedPane)evt.getSource();
        int sel = src.getSelectedIndex();
    }
}
