/*
 *     EncapsulateField.java  Feb 9, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class EncapsulateField extends FieldRefactoring {
    private MethodCode setter = null;
    private MethodCode getter = null;
    private JavaMethod existingSetter;
    private JavaMethod existingGetter;
    private EncapsulateFieldTransformer transformer;
    private SelfEncapsulateFieldTransformer stransformer;

    public EncapsulateField() {
        super();
    }

    public void setUpTest(JFrame f) {
        frame = f;

        JavaField jf = new JavaField(elem);
        int index = 0;
        while (jf.getName().charAt(index) == '_') {
            index++;
        }
        String topChar = jf.getName().substring(index, index + 1);
        String suffixName = topChar.toUpperCase() + jf.getName().substring(index + 1);
        setter = getSetter(jf, "set" + suffixName);
        getter = getGetter(jf, "get" + suffixName);
    }
    
    protected void preconditions() throws RefactoringException {
        if (jfield.isPrivate()) {
            throw new RefactoringException("Not encapsulate: " +
              " the field " + jfield.getName() + " is already private.\n" +
              "Do \"Self Encapsulate Field\"!");
        }

        if (setter == null || getter == null) {
            AccessorDialog dialog = AccessorDialog.show(frame, "Setter/Getter", jfield.getName());
            setter = getSetter(jfield, dialog.getSetterName());
            getter = getGetter(jfield, dialog.getGetterName());
        }
        if (setter == null || getter == null) {
            throw new RefactoringException("");
        }

        existingSetter = jfield.getJavaClass().getJavaMethod(setter.getSignature());
        existingGetter = jfield.getJavaClass().getJavaMethod(getter.getSignature());
        if (existingGetter != null && !existingGetter.getTypeFqn().equals(jfield.getTypeFqn())) {
            throw new RefactoringException("Not encapsulate: " +
              " the return type of the getter " + existingGetter.getName() + " is different from " + jfield.getTypeFqn());
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new EncapsulateFieldTransformer(jfield, setter, getter, existingSetter, existingGetter);
        transformer.changeSrc();
        unparse(jfile);
    }

    protected void additionalTransformation() throws RefactoringException {
        if (!jfield.isPrivate()) {

            stransformer = new SelfEncapsulateFieldTransformer(jfield, setter, getter);
            Iterator it = QueryProject.getAllJavaFilesUsingTheField(jfield).iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();

                if (!jf.equals(jfile)) {
                    stransformer.run(jf);
                    unparse(jf);
                }
            }
        }
    }

    protected String getLog() {
        return jfield.getName() + " through accessors " + getter.getName() + " and " + setter.getName();
    }
}
