/*
 *     PushDownField.java  Mar 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import javax.swing.JFrame;

public class PushDownField extends FieldRefactoring {
    private JavaClass dclass;
    private boolean isStatic = false;
    private PushDownFieldTransformer transformer;

    public PushDownField() {
        super();
    }

    public void setUpTest(JFrame f, JavaClass dclass) {
        frame = f;
        this.dclass = dclass;
    }
    
    protected void preconditions() throws RefactoringException {
        if (QueryClass.getAllChildren(jclass).size() == 0) {
            throw new RefactoringException("Not found:" +
              " children of the class " + jclass.getName());
        }

        if (jfield.isStatic()) {
            isStatic = true;
        }

        if (QueryClass.usesTheField(jclass, jfield)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " is used in the source class " + jclass.getName());
        }

        if (QueryField.usesPrivateFieldsInTheClass(jfield, jclass)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " uses other private fields in the source class " + jclass.getName());
        }

        if (QueryField.callsPrivateMethodsInTheClass(jfield, jclass)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " uses other private fields in the source class " + jclass.getName());
        }

        if (dclass == null) {
            dclass = MoveMemberDialog.show(frame, "Push Down Field: " + jfield.getName(), jclass.getFqn(),
                       SystemProperties.getProjectDir(), QueryClass.getAllChildren(jclass).getArrayList());
        }
        if (dclass == null) {
            throw new RefactoringException("");
        }
        
        if (QueryClass.hasTheField(dclass, jfield.getName())) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " already exists in the target class " + dclass.getName());
        }

        JavaClassList classes = QueryProject.getJavaClassesUsingTheField(jfield);
        classes.remove(jclass);
        classes.removeAll(dclass.getDescendants());
        if (!isStatic && classes.size() != 0) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " is used by non-descendants of the target class" + dclass.getName());
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new PushDownFieldTransformer(jfield);
        transformer.changeSrc();
        unparse(jfile);

        transformer.changeDst(dclass);
        unparse(dclass.getJavaFile());
    }

    protected void additionalTransformation() throws RefactoringException {
        if (isStatic) {
            Iterator it = QueryProject.getAllJavaFilesUsingTheField(jfield).iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();

                transformer.changeRef(jf, dclass.getName());
                unparse(jf);
            }
        }
    }

    protected String getLog() {
        return jfield.getName() + " from " + jclass.getName() + " to " + dclass.getName();
    }
}
