/*
 *     MoveMethod.java  Mar 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import org.w3c.dom.Node;
import javax.swing.JFrame;

public class MoveMethod extends MethodRefactoring {
    private JavaClass dclass;
    private String refName;
    private boolean deleteParam = false;
    private boolean isStatic = false;
    private MoveMethodTransformer transformer;

    public MoveMethod() {
        super();
    }

    public void setUpTest(JFrame f, JavaClass dclass, String refName) {
        frame = f;
        this.dclass = dclass;
        this.refName = refName;
    }

    protected void preconditions() throws RefactoringException {
        if (jmethod.isPrivate() && QueryClass.callsTheMethodExceptSelf(jclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is private and is called by the source class " + jclass.getName());
        }            

        if (jmethod.isAbstract()) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is abstract");
        }

        if (jmethod.isThrows()) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " throws an exception");
        }

        if (QueryMethod.callsMethodsInAncestorsOfTheClassExceptSelf(jmethod, jclass)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " calls a method in the source class " + jclass.getName() + " or its ancestors");
        }

        if (QueryMethod.getWriteJavaFieldsUsingFieldsInAncestorsOfTheClass(jmethod, jclass).size() != 0) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " changes the value of a field in the source class " + jclass.getName() + " or its ancestors");
        }

        if (QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, jclass.getAncestors()).size() != 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " overrides a method in ancestor of the source class " + jclass.getName()); 
        }

        if (QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, jclass.getDescendants()).size() != 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is overridden by a method in descendants of the source class " + jclass.getName());
        }

        if (dclass == null) {
            dclass = MoveMemberDialog.show(frame, "Move Method: " + jmethod.getName(), jclass.getFqn(),
                       SystemProperties.getProjectDir(), QueryProject.getAllJavaClassesInProject().getArrayList());
        }
        if (dclass == null) {
            throw new RefactoringException("");
        }

        if (jclass.getAncestors().contains(dclass)) {
            throw new RefactoringException("Not move:" +
              " the class " + dclass.getName() + " is an ancestor of the source class " + jclass.getName() + ".\n" +
              "Do \"Pull Up Method\"!");
        }

        if (jclass.getDescendants().contains(dclass)) {
            throw new RefactoringException("Not move:" +
              " the class " + dclass.getName() + " is a descendant of the source class " + jclass.getName() + ".\n" +
              "Do \"Push Down Method\"!");
        }

        if (QueryClass.hasTheMethod(dclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " already exists in the target class " + dclass.getName());
        }

        if (!jmethod.isPrivate() && !QueryClass.isAccessible(jclass, dclass)) {
            throw new RefactoringException("Not move:" +
              " the class " + jclass.getName() + " does not access to the target class " + dclass.getName());
        }

        if (!jmethod.isPrivate() && !QueryClass.isAccessible(jclass, dclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the class " + jclass.getName() + " will not access to the moved method " + jmethod.getName());
        }

        JavaFieldList readFields = QueryMethod.getReadJavaFieldsUsingFieldsInAncestorsOfTheClass(jmethod, jclass);
        if (readFields.size() > 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " uses multiple fields in the source class " + jclass.getName());
        }

        if (!QueryClass.isAccessible(dclass, jmethod.getUsedTypes())) {
            throw new RefactoringException("Not move:" +
              " the target class " + dclass.getName() + " will not access to the classes in the method " + jmethod.getName());
        }

        if (jmethod.isStatic()) {
            refName = dclass.getName();
            isStatic = true;

        } else if (readFields.size() == 1) {
            JavaField jf = (JavaField)readFields.get(0);
            if (!jf.getTypeFqn().equals(dclass.getFqn())) {
                throw new RefactoringException("Not move:" +
                  " the method " + jmethod.getName() + " uses a field whose typs is not " + dclass.getName());
            }
            refName = jf.getName();
         
        } else {
            ArrayList fields = collectFieldsOfTheClass(jclass, dclass);
            ArrayList params = collectParametersOfTheClass(jmethod, dclass);

            if (fields.size() == 0 && params.size() == 0) {
                if (QueryProject.getAllJavaFilesCallingTheMethod(jmethod).size() == 0) {
                    refName = null;

                } else {
                    throw new RefactoringException("Not move:" +
                      " no reference to the target class " + dclass.getName() + " can be specified");
                }

            } else if (fields.size() == 1 && params.size() == 0) {
                JavaField f = jclass.getJavaField((String)fields.get(0));
                refName = f.getName();

            } else if (fields.size() == 0 && params.size() == 1) {
                JavaParameter p = jmethod.getParameterNode((String)params.get(0));
                refName = p.getName();
                deleteParam = true;

            } else if (refName != null) {  // for test
                JavaParameter p = jmethod.getParameterNode(refName);
                if (p != null) {
                    deleteParam = true;
                }

            } else {
                String names[] = new String[fields.size() + params.size()];
                for (int i = 0; i < fields.size(); i++) {
                    names[i] = (String)fields.get(i) + " # field";
                }
                for (int i = 0; i < params.size(); i++) {
                    names[i + fields.size()] = (String)params.get(i) + " # parameter";
                }
                String name = ReferenceDialog.show(frame, "Reference name", dclass.getName(), names);

                refName = name.substring(0, name.lastIndexOf("#") - 1);
                if (name.endsWith(" # parameter")) {
                    deleteParam = true;
                }
            }
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new MoveMethodTransformer(jmethod, refName, deleteParam, isStatic);
        transformer.changeSrc();  
        unparse(jfile);

        transformer.changeDst(dclass);
        unparse(dclass.getJavaFile());
    }

    protected void additionalTransformation() throws RefactoringException {
        ArrayList files = QueryProject.getAllJavaFilesCallingTheMethod(jmethod);
        Iterator it = files.iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (isStatic) {
                transformer.changeRef(jf);
            } else {
                CodeHighlighter.run(jf, jmethod);
            }
            unparse(jf);
        }
    }

    protected String getLog() {
        return jmethod.getName() + " from " + jclass.getName() + " to " + dclass.getName();
    }

    private ArrayList collectParametersOfTheClass(JavaMethod jmethod, JavaClass target) {
        ArrayList names = new ArrayList();

        Iterator it = jmethod.getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());

            if (jp.getTypeFqn().equals(target.getFqn())) {
                names.add(jp.getName());
            }
        }

        return names;
    }

    private ArrayList collectFieldsOfTheClass(JavaClass jclass, JavaClass target) {
        ArrayList names = new ArrayList();
        
        Iterator it = jclass.getAllFields().iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();

            if (jf.getTypeFqn().equals(target.getFqn())) {
                names.add(jf.getName());
            }
        }

        return names;
    }
}
