/*
 *     MoveMethodTransformer.java  Feb 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import org.w3c.dom.*;

public class MoveMethodTransformer extends RefactoringTransformer {
    private JavaMethod jmethod;
    private String refName;
    private boolean deleteParam = false;
    private boolean isStatic = false;

    private MoveMethodTransformer() {
    }

    public MoveMethodTransformer(JavaMethod jmethod, String refName, boolean deleteParam, boolean isStatic) {
        this.jmethod = jmethod;
        this.refName = refName;
        this.deleteParam = deleteParam;
        this.isStatic = isStatic;
    }

    public void changeSrc() {
        delete(jmethod);

        if (refName != null && !isStatic) {
            insertDelegationMethod();
        }
    }

    public void changeDst(JavaClass dclass) {
        Node node = insert(jmethod, dclass);

        JavaMethod jm = new JavaMethod(node);
        if (!jmethod.isStatic()) {
            replaceVariableNamesWithThis(jm);
        }
        
        if (deleteParam) {
            deleteRefParameter(jm);
        }
    }

    private void insertDelegationMethod() {
        MethodCode mcode = new MethodCode(jmethod);

        StringBuffer buf = new StringBuffer();
        buf.append(mcode.getCode());
        buf.append(" {\n");

        buf.append("        ");
        if (!jmethod.isVoid()) {
            buf.append("return ");
        }

        buf.append(refName);
        buf.append(".");
        buf.append(jmethod.getName());
        buf.append("(");
        if (deleteParam) {
            buf.append(mcode.getParameterNamesExcept(refName));
        } else {
            buf.append(mcode.getParameterNames());
        }
        buf.append(");\n");
        buf.append("    }\n");

        insertCodeBefore(buf.toString(), jmethod.getElem());
    }

    private void deleteRefParameter(JavaMethod jm) {
        Iterator it = jm.getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());

            if (refName != null && refName.equals(jp.getName())) {
                delete(jp);
            }
        }
    }

    private void replaceVariableNamesWithThis(JavaMethod jm) {
        Iterator it = jm.getUseVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (refName != null && refName.equals(jv.getName())) {
                delete(jv);
            }
        }
    }

    public void changeRef(JavaFile rfile) {
        Iterator it = QueryFile.getJavaCallsUsingTheMethod(rfile, jmethod).iterator();
        while (it.hasNext()) {
            JavaCall ja = (JavaCall)it.next();

            Element elem = (Element)ja.getDotPrefix();
            if (elem != null && elem.getNodeName().equals(JXConstants.TypeElem)) {
                rename(refName, new JavaType(elem));
            } else {
                insertCodeBefore(refName + ".", ja.getElem());
            }
        }
    }
}
