/*
 *     RenameMethod.java  Feb 19, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class RenameMethod extends MethodRefactoring {
    private String newName = null;
    private RenameMethodTransformer transformer;

    public RenameMethod() {
        super();
    }

    public void setUpTest(JFrame f, String newName) {
        frame = f;
        this.newName = newName;
    }

    protected void preconditions() throws RefactoringException {
        if (QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, jclass.getAncestors()).size() != 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " overrides a method in ancestor of the source class " + jclass.getName()); 
        }

        if (QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, jclass.getDescendants()).size() != 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is overridden by a method in descendants of the source class " + jclass.getName());
        }

        if (newName == null) {
            newName = RenameDialog.show(frame, "Rename Method", jmethod.getName());
        }
        if (newName == null) {
            throw new RefactoringException("");
        }

        String signature = jmethod.getSignatureWithTheName(newName);
        if (QueryClass.hasTheMethod(jclass, signature)) {
            throw new RefactoringException("Not rename:" +
              " the method " + newName + " already exists in the class " + jclass.getName());
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new RenameMethodTransformer(jmethod, newName);
        transformer.changeSrc();
        unparse(jfile);
    }

    protected void additionalTransformation() throws RefactoringException {
        ArrayList files = QueryProject.getAllJavaFilesCallingTheMethod(jmethod);
        Iterator it = files.iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            transformer.changeRef(jf);
            unparse(jf);
        }
    }

    protected String getLog() {
        return jmethod.getName() + " to " + newName;
    }
}
