/*
 *     QueryClass.java  Feb 7, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class QueryClass {

    public static String getCanonicalClassName(String name) {
        int sep = name.lastIndexOf("#");
        if (sep != -1) {
            return name.substring(sep + 1);
        }
        return name;
    }

    public static boolean hasTheSameFieldInTheClass(JavaClass jclass1, JavaClass jclass2) {
        Iterator it = jclass1.getAllFields().iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();
            if (hasTheField(jclass2, jf)) {
                return true;
            }
        }
        return false;
    }

    public static boolean hasTheSameMethodInTheClass(JavaClass jclass1, JavaClass jclass2) {
        Iterator it = jclass1.getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            if (hasTheMethod(jclass2, jm)) {
                return true;
            }
        }
        return false;
    }

    public static boolean hasTheField(JavaClass jclass, JavaField jfield) {
        return hasTheField(jclass, jfield.getName());
    }

    public static boolean hasTheField(JavaClass jclass, String fieldName) {
        return jclass.getJavaField(fieldName) != null;
    }

    public static boolean hasTheMethod(JavaClass jclass, JavaMethod jmethod) {
        return hasTheMethod(jclass, jmethod.getSignature());
    }

    public static boolean hasTheMethod(JavaClass jclass, String signature) {
        return jclass.getJavaMethod(signature) != null;
    }
       
    public static JavaClassList getSuperclassesIncludingTheField(JavaClass jclass, JavaField jfield) {
        JavaClassList classes = getClassesIncludingTheField(jclass.getAncestors(), jfield);
        classes.remove(jclass);
        return classes;
    }

    public static JavaClassList collectSubclassesIncludingTheMethod(JavaClass jclass, JavaField jfield) {
        JavaClassList classes = getClassesIncludingTheField(jclass.getDescendants(), jfield);
        classes.remove(jclass);
        return classes;
    }

    public static JavaClassList getClassesIncludingTheField(JavaClassList jclasses, JavaField jfield) {
        JavaClassList classes = new JavaClassList();

        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            if (hasTheField(jc, jfield)) {
                classes.add(jc);
            }
        }
        return classes;
    }
    
    public static JavaClassList getSuperclassesIncludingTheMethod(JavaClass jclass, JavaMethod jmethod) {
        JavaClassList classes = getClassesIncludingTheMethod(jclass.getAncestors(), jmethod);
        classes.remove(jclass);
        return classes;
    }

    public static JavaClassList getSubclassesIncludingTheMethod(JavaClass jclass, JavaMethod jmethod) {
        JavaClassList classes = getClassesIncludingTheMethod(jclass.getDescendants(), jmethod);
        classes.remove(jclass);
        return classes;
    }

    public static JavaClassList getClassesIncludingTheMethod(JavaClassList jclasses, JavaMethod jmethod) {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            if (hasTheMethod(jc, jmethod)) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public static boolean usesTheField(JavaClass jclass, JavaField jfield) {
        return getJavaVariablesUsingTheField(jclass, jfield).size() != 0;
    }
    
    public static JavaVariableList getJavaVariablesUsingTheField(JavaClass jclass, JavaField jfield) {
        return QueryProject.getJavaVariablesUsingTheField(jclass, jfield);
    }

    public static boolean callsTheMethod(JavaClass jclass, JavaMethod jmethod) {
         return getJavaCallsUsingTheMethod(jclass, jmethod).size() != 0;
    }
    
    public static ArrayList getJavaCallsUsingTheMethod(JavaClass jclass, JavaMethod jmethod) {
        return QueryProject.getJavaCallsUsingTheMethod(jclass, jmethod);
    }

    public static boolean usesTheFieldExceptAccessors(JavaClass jclass, JavaField jfield,
                                             String setterSignature, String getterSignature) {
        Iterator it = getJavaVariablesUsingTheField(jclass, jfield).iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            String signature = jv.getJavaMethod().getSignature();
            if (!signature.equals(setterSignature) && !signature.equals(getterSignature)) {
                return true;
            }
        }
        return false;
    }

    public static boolean callsTheMethodExceptSelf(JavaClass jclass, JavaMethod jmethod) {
        Iterator it = getJavaCallsUsingTheMethod(jclass, jmethod).iterator();
        while (it.hasNext()) {
            JavaCall ja = (JavaCall)it.next();
            
            if (!jmethod.equals(ja.getJavaMethod())) {
                return true;
            }
        }
        return false;
    }

    public static JavaClassList getChildrenUsingTheField(JavaClass jclass, JavaField jfield) {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = jclass.getDescendants().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            
            if (jc.isChildOf(jclass) && usesTheField(jc.getDescendants(), jfield)) {
                classes.add(jc);
            }
        }
        return classes;
    }

    private static boolean usesTheField(JavaClassList jclasses, JavaField jfield) {
        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (usesTheField(jc, jfield)) {
                return true;
            }
        }
        return false;
    }

    public static JavaClassList getChildrenCallingTheMethod(JavaClass jclass, JavaMethod jmethod) {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = jclass.getDescendants().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (jc.isChildOf(jclass) && callsTheMethod(jc.getDescendants(), jmethod)) {
                classes.add(jc);
            }
        }
        return classes;
    }

    private static boolean callsTheMethod(JavaClassList jclasses, JavaMethod jmethod) {
        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (callsTheMethod(jc, jmethod)) {
                return true;
            }
        }

        return false;
    }

    public static JavaClassList getAllChildren(JavaClass jclass) {
        JavaClassList classes = new JavaClassList();

        Iterator it = jclass.getDescendants().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (jc.isChildOf(jclass)) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public static JavaClassList getJavaClassesUsedByTheClass(JavaClass jclass) {
        JavaClassList classes = new JavaClassList();

        Iterator it = jclass.getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            if (jt.isObjectRef() || jt.isClassRef()) {
                
                JavaClass jc = jt.getReferredJavaClass();
                if (jc != null) {
                    classes.add(jc);
                }
            }
        }
        return classes;
    }

    private static boolean hasImported(JavaClass jclass, JavaClass target) {
        Iterator it = jclass.getJavaFile().getImportNodes().iterator();
        while (it.hasNext()) {
            JavaImport ji = new JavaImport((Element)it.next());

            if (QueryImport.hasImported(ji, target)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isAccessible(JavaClass jclass, JavaClass target) {
        if (!target.isInTheSamePackage(jclass) &&
            !target.getFqn().startsWith("java.lang") &&
            !hasImported(jclass, target)) {
            return false;
        }

        if (target.isPublic()) {
            return true;
        }

        if (target.isProtected() && jclass.getAncestors().contains(target)) {
            return true;
        }

        if (!target.isPrivate() && target.isInTheSamePackage(jclass)) {
            return true;
        }
        return false;
    }

    public static boolean isAccessible(JavaClass jclass, JavaClass dclass, JavaMethod target) {
        if (!isAccessible(jclass, dclass)) {
            return false;
        }

        if (target.isPublic()) {
            return true;
        }

        if (target.isProtected() && jclass.getAncestors().contains(dclass)) {
            return true;
        }

        if (!target.isPrivate() && dclass.isInTheSamePackage(jclass)) {
            return true;
        }
        return false;
    }

    public static boolean isAccessible(JavaClass jclass, JavaClass dclass, JavaField target) {
        if (!isAccessible(jclass, dclass)) {
            return false;
        }

        if (target.isPublic()) {
            return true;
        }

        if (target.isProtected() && jclass.getAncestors().contains(dclass)) {
            return true;
        }

        if (!target.isPrivate() && dclass.isInTheSamePackage(jclass)) {
            return true;
        }
        return false;
    }

    public static boolean isAccessible(JavaClass jclass, JavaClassList classes) {
        Iterator it = classes.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (!isAccessible(jclass, jc)) {
                return false;
            }
        }
        return true;
    }
}
