/*
 *     QueryField.java  Feb 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import org.w3c.dom.*;

public class QueryField {

    public static boolean usesOtherFields(JavaField jfield) {
        return jfield.getUseVariables().size() != 0;
    }

    public static boolean callsOtherMethods(JavaField jfield) {
        Iterator it = jfield.getMethodCallNodes().iterator();
        while (it.hasNext()) {
            JavaCall ja = new JavaCall((Node)it.next());

            JavaMethod jm = ja.getCalledJavaMethod();
            if (jm != null && ja.isMethodCall()) {
                return true;
            }
        }
        return false;
    }

    public static boolean usesFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        return getJavaFieldsUsingFieldsInTheClass(jfield, jclass).size() != 0;
    }
    
    public static boolean usesFieldsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (usesFieldsInTheClass(jfield, jc)) {
                return true;
            }
        }
        return false;
    }
    
    public static JavaFieldList getJavaFieldsUsingFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        JavaVariableList uses = jfield.getUseVariables();
        Iterator it = uses.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            JavaField jf = jv.getUsedJavaField();
            if (jf != null && jclass.equals(jf.getJavaClass())) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static boolean callsMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        return getJavaMethodsCallingMethodsInTheClass(jfield, jclass).size() != 0;
    }
    
    public static boolean callsMethodsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (callsMethodsInTheClass(jfield, jc)) {
                return true;
            }
        }
        return false;
    }
    
    public static JavaMethodList getJavaMethodsCallingMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        JavaMethodList methods = new JavaMethodList();

        Iterator it = jfield.getMethodCallNodes().iterator();
        while (it.hasNext()) {
            JavaCall ja = new JavaCall((Node)it.next());

            JavaMethod jm = ja.getCalledJavaMethod();
            if (jm != null && ja.isMethodCall() && jclass.equals(jm.getJavaClass())) {
                methods.add(jm);
            }
        }
        return methods;
    }

    public static boolean usesPrivateFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = getJavaFieldsUsingFieldsInTheClass(jfield, jclass).iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();

            if (jf.isPrivate()) {
                return true;
            }
        }
        return false;
    }

    public static boolean callsPrivateMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = getJavaMethodsCallingMethodsInTheClass(jfield, jclass).iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (jm.isPrivate()) {
                return true;
            }
        }
        return false;
    }

    public static JavaFieldList getJavaFieldsUsingFieldsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            fields.addAll(getJavaFieldsUsingFieldsInTheClass(jfield, jc));
        }

        return fields;
    }

    public static JavaFieldList getJavaFieldsWithTheSameName(JavaField jfield, JavaClassList jclasses) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            JavaField jf = jc.getJavaField(jfield.getName());
            if (jf != null) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static boolean isInOnlySimpleAssignments(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (!isInOnlySimpleAssignments(jfield, jm)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isInOnlySimpleAssignments(JavaField jfield, JavaMethod jmethod) {
        Iterator it = jmethod.getDefVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            JavaField jf = jv.getUsedJavaField();
            if (jf != null && !isInSimpleAssignment(jv)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isInSimpleAssignment(JavaVariable jv) {
        Node node = jv.getElem().getParentNode();
        if (node != null && node.getNodeName().equals(JXConstants.ExpressionElem)) {
            JavaExpr je = new JavaExpr(node);

            if (je.isInSimpleAssignment()) {
                return true;
            }
        }

        Element elem = (Element)jv.getDotPrefix();
        if (elem != null && elem.getNodeName().equals(JXConstants.ExpressionElem) &&
            elem.getAttribute(JXConstants.SortAttr).equals(JXConstants.ThisRef)) {
            return true;
        }

        return false;
    }
}
