/*
 *     QueryMethod.java  Feb 6, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class QueryMethod {

    public static boolean callsMethodsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        return getJavaMethodsCallingMethodsInTheClass(jmethod, jclass).size() != 0;
    }
    
    public static boolean callsMethodsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (callsMethodsInTheClass(jmethod, jc)) {
                return true;
            }
        }

        return false;
    }

    public static boolean callsMethodsInTheClassExceptSelf(JavaMethod jmethod, JavaClass jclass) {
        JavaMethodList methods = getJavaMethodsCallingMethodsInTheClass(jmethod, jclass);
        methods.remove(jmethod);
        return methods.size() != 0;
    }

    public static boolean callsMethodsInAncestorsOfTheClassExceptSelf(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (callsMethodsInTheClassExceptSelf(jmethod, jc)) {
                return true;
            }
        }
        return false;
    }

    public static JavaMethodList getJavaMethodsCallingMethodsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaMethodList methods = new JavaMethodList();

        Iterator it = jmethod.getMethodCallNodes().iterator();
        while (it.hasNext()) {
            JavaCall ja = new JavaCall((Node)it.next());
            
            JavaMethod jm = ja.getCalledJavaMethod();
            if (jm != null && jclass.equals(jm.getJavaClass())) {
                methods.add(jm);
            }                
        }
        return methods;
    }

    public static JavaMethodList getJavaMethodsCallingMethodsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaMethodList methods = new JavaMethodList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            methods.addAll(getJavaMethodsCallingMethodsInTheClass(jmethod, jc));
        }

        return methods;
    }


    public static boolean usesFieldsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        return getJavaFieldsUsingFieldsInTheClass(jmethod, jclass).size() != 0;
    }

    public static boolean usesFieldsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (usesFieldsInTheClass(jmethod, jc)) {
                return true;
            }
        }
        return false;
    }

    public static JavaFieldList getJavaFieldsUsingFieldsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jmethod.getVariableAccessNodes().iterator();
        while (it.hasNext()) {
            JavaVariable jv = new JavaVariable((Node)it.next());

            JavaField jf = jv.getUsedJavaField();
            if (jf != null && jclass.equals(jf.getJavaClass())) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static JavaFieldList getJavaFieldsUsingFieldsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            fields.addAll(getJavaFieldsUsingFieldsInTheClass(jmethod, jc));
        }

        return fields;
    }

    public static JavaFieldList getReadJavaFieldsUsingFieldsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            fields.addAll(getReadJavaFieldsUsingFieldsInTheClass(jmethod, jc));
        }

        return fields;
    }

    public static JavaFieldList getReadJavaFieldsUsingFieldsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jmethod.getUseVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            JavaField jf = jv.getUsedJavaField();
            if (jf != null && jclass.equals(jf.getJavaClass())) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static JavaFieldList getWriteJavaFieldsUsingFieldsInAncestorsOfTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            fields.addAll(getWriteJavaFieldsUsingFieldsInTheClass(jmethod, jc));
        }

        return fields;
    }

    public static JavaFieldList getWriteJavaFieldsUsingFieldsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jmethod.getDefVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            JavaField jf = jv.getUsedJavaField();
            if (jf != null && jclass.equals(jf.getJavaClass())) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static boolean callsPrivateMethodsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = getJavaMethodsCallingMethodsInTheClass(jmethod, jclass).iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (jm.isPrivate()) {
                return true;
            }
        }
        return false;
    }

    public static boolean callsPrivateMethodsInTheClassExceptSelf(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = getJavaMethodsCallingMethodsInTheClass(jmethod, jclass).iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (jm.isPrivate() && !jm.equals(jmethod)) {
                return true;
            }
        }
        return false;
    }

    public static boolean usesPrivateFieldsInTheClass(JavaMethod jmethod, JavaClass jclass) {
        Iterator it = getJavaFieldsUsingFieldsInTheClass(jmethod, jclass).iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();

            if (jf.isPrivate()) {
                return true;
            }
        }

        return false;
    }

    public static boolean usesThePrivateField(JavaMethod jmethod, JavaField jfield) {
        Iterator it =getJavaFieldsUsingFieldsInTheClass(jmethod, jfield.getJavaClass()).iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();

            if (jf.equals(jfield) && jf.isPrivate()) {
                return true;
            }
        }

        return false;
    }

    public static boolean hasTheVariable(JavaMethod jmethod, String varName) {
        Iterator it = jmethod.getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            if (jv.getName().equals(varName)) {
                return true;
            }
        }
        return false;
    }

    public static boolean hasTheLocalVariable(JavaMethod jmethod, String varName) {
        Iterator it = jmethod.getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            if (!jv.isField() && jv.getName().equals(varName)) {
                return true;
            }
        }
        return false;
    }

    public static boolean usesTheVariable(JavaMethod jmethod, String varName) {
        JavaVariableList defs = jmethod.getDefVariableList();
        JavaVariableList uses = jmethod.getUseVariableList();

        JavaVariableList vars = new JavaVariableList();
        vars.addAll(defs);
        vars.addAll(uses);

        Iterator it = vars.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            if (jv.getName().equals(varName)) {
                return true;
            }
        }
        return false;
    }

    public static JavaMethodList getJavaMethodsWithTheSameSignature(JavaMethod jmethod, JavaClassList jclasses) {
        JavaMethodList methods = new JavaMethodList();

        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            JavaMethod jm = jc.getJavaMethod(jmethod.getSignature());
            if (jm != null) {
                methods.add(jm);
            }
        }
        return methods;
    }
}
