/*
 *     SwitchToPolymorphismTransformer.java  Feb 23, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.statements;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import org.w3c.dom.*;

public class SwitchToPolymorphismTransformer extends RefactoringTransformer {
    private JavaStatement jstatement;
    private ArrayList dclasses = new ArrayList();  // DstJavaClass
    private HashMap codeMap = new HashMap();
    private CFG cfg;

    public SwitchToPolymorphismTransformer(JavaStatement jstatement, ArrayList dclasses) {
        this.jstatement = jstatement;
        this.dclasses = dclasses;

        cfg = new CFG(jstatement.getJavaMethod().getElem());
        // cfg.print();
    }
    
    public void changeSrc() {
        attachModifier(jstatement.getJavaClass(), "abstract ");
        insertAbstractMethod(jstatement.getJavaMethod());
        delete(jstatement.getJavaMethod());

        createMethodCodes(jstatement);
        createNewSubClasses(jstatement);
    }

    public void changeDst(JavaClass dclass, String label) {
        String code = getNewMethodCode(label, jstatement);
        insertCodeBefore("\n" + code, dclass.getLastMemeberNode());
    }

    private void attachModifier(JavaClass jc, String modifier) {
        Node node = jc.getElem().getFirstChild();
        insertCodeBefore(modifier, node);
    }

    private void insertAbstractMethod(JavaMethod jm) {
        MethodCode mcode = new MethodCode(jm);
        mcode.removeModifier(Modifier.PRIVATE);

        StringBuffer buf = new StringBuffer();
        buf.append("abstract ");
        buf.append(mcode.getCode());
        buf.append(";\n");
        insertCodeBefore(buf.toString(), jm.getElem());
    }

    private void createNewSubClasses(JavaStatement jst) {
        Iterator it = dclasses.iterator();
        while (it.hasNext()) {
            DstJavaClass dclass = (DstJavaClass)it.next();

            JavaClass jc = dclass.getJavaClass();
            if (jc == null) {
                String code = getNewSubclassCode(dclass.getLabel(), dclass.getName(), jst);
                insertCode(code, jst.getJavaFile().getElem());
            }
        }
    }

    private String getNewSubclassCode(String label, String name, JavaStatement jst) {
        StringBuffer buf = new StringBuffer();

        buf.append("\n");
        buf.append("class ");
        buf.append(name);
        buf.append(" extends ");
        buf.append(jst.getJavaClass().getName());

        buf.append(" {\n");
        buf.append(getNewMethodCode(label, jst));
        buf.append("}\n");

        return buf.toString();
    }

    private String getNewMethodCode(String label, JavaStatement jst) {
        MethodCode mcode = new MethodCode(jst.getJavaMethod());
        mcode.removeModifier(Modifier.PRIVATE);

        StringBuffer buf = new StringBuffer();
        buf.append("    ");
        buf.append(mcode.getCode());
        buf.append(" {\n");
        buf.append((String)codeMap.get(label));
        buf.append("    }\n");

        return buf.toString();
    }

    private CFGNode switchNode;
    private ArrayList caseElements = new ArrayList();  // Element

    private void createMethodCodes(JavaStatement jst) {
        switchNode = cfg.getNode(jst.getElem());

        NodeList nl = jst.getElem().getElementsByTagName(JXConstants.LabelElem);
        for (int i = 0; i < nl.getLength(); i++) {
            JavaStatement js = new JavaStatement(nl.item(i));

            String label;
            if (js.getKeyword(js.getElem()).equals("case")) {
                Node node = js.getFirstChildNode(js.getElem(), JXConstants.ExpressionElem);
                label = DOM2Code.getCode(node);
            } else {  // js.getKeyword(elem).equals("default")
                label = "default";
            }

            CFGNode caseNode = cfg.getNode(js.getElem());
            collectCaseNodes(caseNode);

            StringBuffer buf = new StringBuffer();
            Iterator it = caseElements.iterator();
            while(it.hasNext()) {
                Element elem = (Element)it.next();

                buf.append(getContentsOfCode(elem));
                buf.append("\n");
            }

            codeMap.put(label, buf.toString());
        }
    }

    private void collectCaseNodes(CFGNode cfgNode) {
        caseElements.clear();

        Iterator it = cfgNode.getOutgoingEdges().iterator();
        while (it.hasNext()) {
            Flow flow = (Flow)it.next();
            if (flow.isTrue()) {
                traverseForwardCF((CFGNode)flow.getDstNode());
            }
        }
    }    

    private void traverseForwardCF(CFGNode cfgNode) {
        Element elem = cfgNode.getElem();
        if ((elem != null && elem.getNodeName().equals(JXConstants.StatementElem)) ||
            cfgNode.isAssignmentSt()) {

            if (!contains(caseElements, elem)) {
                caseElements.add(elem);
            }
        }

        if (isCaseEnd(cfgNode)) {
            return;
        }

        Iterator it = cfgNode.getOutgoingEdges().iterator();
        while (it.hasNext()) {
            Flow flow = (Flow)it.next();

            if (flow.isTrue() || flow.isFalse()) {
                CFGNode succ = (CFGNode)flow.getDstNode();
                if (succ != null) {
                    traverseForwardCF(succ);
                }
            }
        }
    }

    private boolean contains(ArrayList elems, Node node) {
        while (node != null) {
            if (elems.contains(node)) {
                return true;
            }
            
            node = node.getParentNode();
        }
        return false;
    }


    private boolean isCaseEnd(CFGNode cfgNode) {
        if (cfgNode.isReturnSt()) {
            return true;
        }

        if (cfgNode.isMergeSt()) {
            CFGMerge merge = (CFGMerge)cfgNode;
            if (switchNode.equals(merge.getBranchNode())) {

                int last = caseElements.size() - 1;
                Element elem = (Element)caseElements.get(last);
                if (elem.getNodeName().equals(JXConstants.StatementElem)) {
                    JavaStatement jst = new JavaStatement(elem);
                    if (jst.isBreakStatement()) {
                        caseElements.remove(last);
                    }
                }
                return true;
            }
        }
        return false;
    }
}
