/*
 *     MethodCode.java  Dec 3, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class MethodCode {
    private Modifier modifier = new Modifier();
    private String type;
    private String name;
    private ArrayList parameters = new ArrayList();

    private MethodCode() {
    }

    public MethodCode(String type, String name) {
        this.type = type;
        this.name = name;
    }

    public MethodCode(JavaMethod jm) {
        this(jm.getType(), jm.getName());

        Iterator it = jm.getParameterNodes().iterator();
        while (it.hasNext()) {
            JavaParameter jp = new JavaParameter((Node)it.next());
            addParameter(jp.getType(), jp.getTypeFqn(), jp.getName());
        }

        if (jm.isPublic()) {
            addModifier(Modifier.PUBLIC);

        } else if (jm.isProtected()) {
            addModifier(Modifier.PROTECTED);

        } else if (jm.isPrivate()) {
            addModifier(Modifier.PRIVATE);

        } else if (jm.isAbstract()) {
            addModifier(Modifier.ABSTRACT);

        } else if (jm.isStatic()) {
            addModifier(Modifier.STATIC);

        } else if (jm.isFinal()) {
            addModifier(Modifier.FINAL);

        } else if (jm.isSynchronized()) {
            addModifier(Modifier.SYNCHRONIZED);
        }
    }

    public void addModifier(String keyword) {
        modifier.add(keyword);
    }

    public void addModifier(int index, String keyword) {
        modifier.add(index, keyword);
    }

    public void removeModifier(String keyword) {
        modifier.remove(keyword);
    }

    public void addParameter(String ptype, String pfqn, String pname) {
        ParamCode param = new ParamCode(ptype, pfqn, pname);
        parameters.add(param);
    }

    public void removeParameter(String pname) {
        ArrayList tmpPparameters = new ArrayList(parameters);

        Iterator it = tmpPparameters.iterator();
        while (it.hasNext()) {
            ParamCode param = (ParamCode)it.next();
            if (param.getName().equals(pname)) {
                parameters.remove(param);
            }
        }
    }

    public boolean equals(JavaMethod jm) {
        return getSignature().equals(jm.getSignature());
    }

    public String getName() {
        return name;
    }

    public String getType() {
        return type;
    }

    public String getSignature() {
        return getName() + "(" + getParameterTypes() + " )";
    }

    private String getParameterTypes() {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < parameters.size(); i++) {
            buf.append(" ");
            ParamCode param = (ParamCode)parameters.get(i);
            buf.append(param.getTypeFQN());
        }
        return buf.toString();
    }

    public String getParameterName(int index) {
        if (index < parameters.size()) {
            ParamCode param = (ParamCode)parameters.get(index);
            return param.getName();
        }
        return "";
    }

    public String getParameterNames() {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < parameters.size(); i++) {
            if (i != 0) {
                buf.append(", ");
            }
            ParamCode param = (ParamCode)parameters.get(i);
            buf.append(param.getName());
        }
        return buf.toString();
    }

    public String getParameterNamesExcept(String pname) {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < parameters.size(); i++) {
            if (i != 0) {
                buf.append(", ");
            }
            ParamCode param = (ParamCode)parameters.get(i);
            if (!param.getName().equals(pname)) {
                buf.append(param.getName());
            }
        }
        return buf.toString();
    }

    public String getCode() {
        StringBuffer buf = new StringBuffer();

        buf.append(modifier.toString());
        if (type != null) {
            if (buf.length() != 0) {
                buf.append(" ");
            }
            buf.append(type);
        }

        if (buf.length() != 0) {
            buf.append(" ");
        }

        buf.append(name);
        buf.append("(");
        for (int i = 0; i < parameters.size(); i++) {
            if (i != 0) {
                buf.append(", ");
            }
            ParamCode param = (ParamCode)parameters.get(i);
            buf.append(param.toString());
        }
        buf.append(")");

        return buf.toString();
    }
}

class ParamCode {
    private String type;
    private String fqn;
    private String name;

    ParamCode(String type, String fqn, String name) {
        this.type = type;
        this.fqn = fqn;
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public String getTypeFQN() {
        return fqn;
    }

    public String getName() {
        return name;
    }

    public String toString() {
        return type + " " + name;
    }
}
