/*
 *     RefactoringTransformer.java  Mar 25, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.dom.*;
import java.util.*;
import org.w3c.dom.*;

public class RefactoringTransformer {

    protected RefactoringTransformer() {
    }

    protected Node rename(String newText, JavaClass jclass) {
        Node node = jclass.getIdentNode();
        return rename(newText, node);
    }

    protected Node rename(String newText, JavaMethod jmethod) {
        Node node = jmethod.getIdentNode();
        return rename(newText, node);
    }

    protected Node rename(String newText, JavaField jfield) {
        Node node = jfield.getIdentNode();
        return rename(newText, node);
    }

    protected Node rename(String newText, JavaType jtype) {
        Node node = jtype.getFirstChildNode(jtype.getElem(), JXConstants.IdentElem);
        return rename(newText, node);
    }
    
    protected Node rename(String newText, JavaCall jcall) {
        Node node = jcall.getFirstChildNode(jcall.getElem(), JXConstants.IdentElem);
        return rename(newText, node);
    }
    
    protected Node rename(String newText, JavaVariable jvar) {
        Node node = jvar.getFirstChildNode(jvar.getElem(), JXConstants.IdentElem);
        return rename(newText, node);
    }
           
    protected Node rename(String newText, Node node) {
        deleteText(node);
        Node newNode = insertNode(newText, node);

        return newNode;
    }

    protected void deleteText(Node node) {
        delete(node);
    }

    protected Node insertNode(String text, Node node) {
        Element newNode = node.getOwnerDocument().createElement(node.getNodeName());
        Node newText = node.getOwnerDocument().createTextNode(text);
        newNode.appendChild(newText);
        insertAfter(newNode, node);

        newNode.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    protected void delete(JavaClass jc) {
        delete(jc.getElem());
    }

    protected Node insert(JavaClass jc, JavaFile jf) {
        insertCode("\n", jf.getElem());
        return insertElement(jc, jf.getElem());
    }

    protected void delete(JavaMethod jm) {
        delete(jm.getElem());
    }

    protected Node insert(JavaMethod jm, JavaClass jc) {
        Node node = jc.getLastMemeberNode();
        insertCodeBefore("\n", node);
        insertCodeBefore("    ", node);

        Node newNode = insertBefore(jm, jc.getLastMemeberNode());
        insertCodeBefore("\n", node);

        return newNode;
    }

    protected void delete(JavaField jf) {
        if (jf.getNumFieldInDeclaration() == 1) {
            delete(jf.getElem());
            
            Node node = jf.getElem().getNextSibling();
            while (!node.getNodeName().equals(JXConstants.OperatorElem)) {
                delete(node);
                node = node.getNextSibling();
            }
            if (node.getFirstChild().getNodeValue().equals(";")) {
                delete(node);
            }

        } else {
            delete(jf.getIdentNode());

            if (jf.isOmitted()) {
                Node node = jf.getElem().getPreviousSibling();
                while (!node.getNodeName().equals(JXConstants.FieldElem)) {
                    delete(node);
                    node = node.getPreviousSibling();
                }

            } else {
                Node node = jf.getElem().getNextSibling();
                while (!node.getNodeName().equals(JXConstants.FieldElem)) {
                    delete(node);
                    node = node.getNextSibling();
                }
            }
        }
    }

    protected Node insert(JavaField jf, JavaClass jc) {
        Node node = jc.getLastMemeberNode();
        insertCodeBefore("\n", node);
        insertCodeBefore("    ", node);

        JavaField tf = jf.getFieldWithDeclaration();
        Node newNode = insertBefore(tf, node);

        JavaField f = new JavaField(newNode);
        Node n = f.getIdentNode().getFirstChild();
        n.setNodeValue(jf.getName());

        insertCodeBefore(";", node);
        insertCodeBefore("\n", node);

        return newNode;
    }

    protected void delete(JavaParameter jp) {
        delete(jp.getElem());

        JavaMethod jm = jp.getJavaMethod();
        if (jm.getNumParameter() != 1) {

            if (jm.getParameterIndex(jp) == 0) {
                Node node = jp.getElem().getNextSibling();
                while (!node.getNodeName().equals(JXConstants.ParameterElem)) {
                    delete(node);
                    node = node.getNextSibling();
                }

            } else {
                Node node = jp.getElem().getPreviousSibling();
                while (!node.getNodeName().equals(JXConstants.ParameterElem)) {
                    delete(node);
                    node = node.getPreviousSibling();
                }
            }
        }
    }

    protected void delete(JavaCall ja, int i) {
        ArrayList arguments = ja.getArgumentNodes();
        Node node = (Node)arguments.get(i);
        delete(node);

        if (ja.getNumArgument() != 1) {
            if (i == 0) {
                node = node.getNextSibling();
                while (!node.getNodeName().equals(JXConstants.ExpressionElem)) {
                    delete(node);
                    node = node.getNextSibling();
                }
            } else {
                node = node.getPreviousSibling();
                while (!node.getNodeName().equals(JXConstants.ExpressionElem)) {
                    delete(node);
                    node = node.getPreviousSibling();
                }
            }
        }            
    }

    protected void delete(JavaLocal jl) {
        if (jl.getNumLocalInDeclaration() == 1) {
            delete(jl.getElem());
            
            Node node = jl.getElem().getNextSibling();
            while (!node.getNodeName().equals(JXConstants.OperatorElem)) {
                delete(node);
                node = node.getNextSibling();
            }

            if (node.getFirstChild().getNodeValue().equals(";")) {
                delete(node);
            }

        } else {
            delete(jl.getIdentNode());
            if (jl.isOmitted()) {
                Node node = jl.getElem().getPreviousSibling();
                while (!node.getNodeName().equals(JXConstants.LocalElem)) {
                    delete(node);
                    node = node.getPreviousSibling();
                }

            } else {
                Node node = jl.getElem().getNextSibling();
                while (!node.getNodeName().equals(JXConstants.LocalElem)) {
                    delete(node);
                    node = node.getNextSibling();
                }
            }
        }
    }

    protected void delete(JavaVariable jv) {
        delete(jv.getElem());

        Element elem = (Element)jv.getElem().getNextSibling();
        if (elem.getNodeName().equals(JXConstants.OperatorElem)) {
            Node node = elem.getFirstChild();
            if (node.getNodeValue().equals(".")) {
                delete(elem);
            }
        }
    }

    protected Node insert(JavaImport ji, JavaFile jf) {
        Node newNode;
        ArrayList imports = jf.getImportNodes();
        if (imports.size() != 0) {
            Node node = (Node)imports.get(imports.size() - 1);
            newNode = insertAfter(ji, node);

        } else if (jf.getPackageName().length() != 0) {
            newNode = insertAfter(ji, jf.getPackageNode());

        } else {
            ArrayList classes = jf.getClassNodes();
            if (classes.size() != 0) {
                Node node = (Node)classes.get(0);
                newNode = insertBefore(ji, node);

            } else {
                newNode = insertElement(ji, jf.getElem());
            }
        }
        insertCodeBefore("\n", newNode);
        return newNode;
    }

    protected Node insertImportCode(String name, JavaFile jf) {
        String code = "import " + name + ";";
        Node newNode;
        ArrayList imports = jf.getImportNodes();
        if (imports.size() != 0) {
            Node node = (Node)imports.get(imports.size() - 1);
            newNode = insertCodeAfter(code, node);

        } else if (jf.getPackageName().length() != 0) {
            newNode = insertCodeAfter(code, jf.getPackageNode());

        } else {
            ArrayList classes = jf.getClassNodes();
            if (classes.size() != 0) {
                Node node = (Node)classes.get(0);
                newNode = insertCodeBefore(code, node);

            } else {
                newNode = insertCode(code, jf.getElem());
            }
        }
        insertCodeBefore("\n", newNode);
        return newNode;
    }

    protected void delete(JavaImport ji) {
        delete(ji.getElem());
    }

    protected void delete(Node node) {
        Element elem = (Element)node;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.DELETED);
    }

    private Node insertElement(JavaElement jelem, Node node) {
        Node newNode = node.getOwnerDocument().importNode(jelem.getElem(), true);
        node.appendChild(newNode);

        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    protected Node insertCode(String code, Node node) {
        Node newNode = insertCodeWithoutHighlight(code, node);

        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    private Node insertCodeWithoutHighlight(String code, Node node) {
        Element newNode = node.getOwnerDocument().createElement(RefactoringConstants.CodeElem);
        Node newText = node.getOwnerDocument().createTextNode(code);
        newNode.appendChild(newText);
        node.appendChild(newNode);

        return newNode;
    }

    protected Node insertCodeBefore(String code, Node node) {
        Node newNode = insertCodeBeforeWithoutHighlight(code, node);

        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    private Node insertCodeBeforeWithoutHighlight(String code, Node node) {
        Element newNode = node.getOwnerDocument().createElement(RefactoringConstants.CodeElem);
        Node newText = node.getOwnerDocument().createTextNode(code);
        newNode.appendChild(newText);
        insertBefore(newNode, node);

        return newNode;
    }

    private void insertBefore(Node newNode, Node node) {
        node.getParentNode().insertBefore(newNode, node);
    }

    protected Node insertCodeAfter(String code, Node node) {
        Node newNode = insertCodeAfterWithoutHighlight(code, node);
        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    private Node insertCodeAfterWithoutHighlight(String code, Node node) {
        Element newNode = node.getOwnerDocument().createElement(RefactoringConstants.CodeElem);
        Node newText = node.getOwnerDocument().createTextNode(code);
        newNode.appendChild(newText);
        insertAfter(newNode, node);

        return newNode;
    }

    private void insertAfter(Node newNode, Node node) {
        Node sibling = node.getNextSibling();
        if (sibling != null) {
            node.getParentNode().insertBefore(newNode, sibling);
        } else {
            node.getParentNode().appendChild(newNode);
        }
    }

    private Node insertBefore(JavaElement jelem, Node node) {
        Node newNode = node.getOwnerDocument().importNode(jelem.getElem(), true);
        insertBefore(newNode, node);

        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    private Node insertAfter(JavaElement jelem, Node node) {
        Node newNode = node.getOwnerDocument().importNode(jelem.getElem(), true);
        insertAfter(newNode, node);

        Element elem = (Element)newNode;
        elem.setAttribute(RefactoringConstants.ChangeAttr, RefactoringConstants.INSERTED);

        return newNode;
    }

    protected void changeAccessModifier(JavaClass jc, String modifier) {
        changeAccessModifier(jc, modifier, jc.isDefault());
    }

    protected void changeAccessModifier(JavaMethod jm, String modifier) {
        changeAccessModifier(jm, modifier, jm.isDefault());
    }

    protected void changeAccessModifier(JavaField jf, String modifier) {
        changeAccessModifier(jf, modifier, jf.isDefault());
    }

    private void changeAccessModifier(JavaElement jelem, String modifier, boolean isDefault) {
        if (isDefault) {
            Node node = jelem.getElem().getFirstChild();
            insertCodeBefore(modifier + " ", node);
        } else {
            Iterator it = jelem.getDescendantNodes(jelem.getElem(), JXConstants.KeywordElem).iterator();
            while (it.hasNext()) {
                Element elem = (Element)it.next();

                String keyword = elem.getFirstChild().getNodeValue();
                if (keyword.equals("public") || keyword.equals("protected") || keyword.equals("private")) {
                    rename(modifier, elem);
                }
            }
        }
    }

    protected void renameDotPrefix(JavaFile jfile, JavaMethod jmethod, String name) {
        Iterator it = QueryFile.getJavaCallsUsingTheMethod(jfile, jmethod).iterator();
        while (it.hasNext()) {
            JavaCall ja = (JavaCall)it.next();

            Element elem = (Element)ja.getDotPrefix();
            if (elem != null && elem.getNodeName().equals(JXConstants.TypeElem)) {
                rename(name, new JavaType(elem));
            }
        }
    }

    protected void renameDotPrefix(JavaFile jfile, JavaField jfield, String name) {
        Iterator it = QueryFile.getJavaVariablesUsingTheField(jfile, jfield).iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            Element elem = (Element)jv.getDotPrefix();
            if (elem != null && elem.getNodeName().equals(JXConstants.TypeElem)) {
                rename(name, new JavaType(elem));
            }
        }
    }

    protected void eliminateDotPrefix(JavaClass jclass, JavaMethod jmethod, String prefix) {
        Iterator it = QueryClass.getJavaCallsUsingTheMethod(jclass, jmethod).iterator();
        while (it.hasNext()) {
            JavaCall ja = (JavaCall)it.next();

            Element elem = (Element)ja.getDotPrefix();
            eliminateDotPrefix(elem, prefix);
        }
    }

    protected void eliminateDotPrefix(JavaClass jclass, JavaField jfield, String prefix) {
        Iterator it = QueryClass.getJavaVariablesUsingTheField(jclass, jfield).iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            Element elem = (Element)jv.getDotPrefix();
            eliminateDotPrefix(elem, prefix);
        }
    }

    protected boolean eliminateDotPrefix(Element elem, String prefix) {
        if (elem != null) {
            if (prefix.equals("this")) {
                if (elem.getNodeName().equals(JXConstants.ExpressionElem) &&
                    elem.getAttribute(JXConstants.SortAttr).equals(JXConstants.ThisRef)) {
                    delete(elem);
                    delete(elem.getNextSibling());
                    return true;
                }

            } else {
                if (elem.getNodeName().equals(JXConstants.KeywordElem)) {
                    String str = elem.getFirstChild().getNodeValue();
                    if (str.equals(prefix)) {
                        delete(elem);
                        delete(elem.getNextSibling());
                        return true;
                    }
                }
            }
        }
        return false;
    }

    protected String getContentsOfCode(Element elem) {
        String code = getSpace(elem) + DOM2Code.getCode(elem);
        code = removeIndent(code);
        return code;
    }

    protected String getSpace(Element elem) {
        Node node = elem.getPreviousSibling();
        if (node != null && node.getNodeName().equals(JXConstants.SpaceElem)) {
            return node.getFirstChild().getNodeValue();
        }
        return "";
    }

    protected String removeIndent(String text) {
        int num = 0;
        StringTokenizer st = new StringTokenizer(text, "\n", false);
        while (st.hasMoreTokens()) {
            String line = st.nextToken();
            int n = countSpacesAtIndent(line);
            if (num == 0 || n < num) {
                num = n;
            }
        }
        
        StringBuffer buf = new StringBuffer();
        st = new StringTokenizer(text, "\n", true);
        while (st.hasMoreTokens()) {
            String line = st.nextToken();
            if (line.length() > num) {
                buf.append("        " + line.substring(num));
            } else {
                buf.append(line);
            }
        }
        return buf.toString();
    }

    protected int countSpacesAtIndent(String text) {
        int count = 0;
        while (text.charAt(count) == ' ') {
            count++;
        }
        return count;
    }
}
