/*
 *     SplitVariable.java  Feb 12, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.variables;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.graph.pdg.*;
import java.util.*;
import org.w3c.dom.*;
import javax.swing.JFrame;

public class SplitVariable extends VariableRefactoring {
    private String newName = null;
    private JavaVariable jv;
    private SplitVariableTransformer transformer;
    private CFG cfg;
    private PDG pdg;
    private GraphCompSet nodesToBeChanged = new GraphCompSet();

    public SplitVariable() {
        super();
    }

    public void setUpTest(JFrame f, String newName) {
        frame = f;
        this.newName = newName;
    }

    protected void preconditions() throws RefactoringException {
        Element selectedElem = (Element)selectedElems.get(0);
        jv = new JavaVariable(selectedElem.getParentNode());
        cfg = new CFG(jv.getJavaMethod().getElem());
        pdg = new PDG(cfg);

        CFGNode node = getCFGNode(jv);
        if (node == null) {
            throw new RefactoringException("Invalid refactoring");
        }
        collectNodesToBeChanged(pdg.getNode(node));
        if (!possibleToSplit(nodesToBeChanged)) {
            throw new RefactoringException("Not split:" +
              " the variable " + newName + " is imposible to split");
        }

        if (newName == null) {
            newName = RenameDialog.show(frame, "Split Variable", jvar.getName());
        }
        if (newName == null) {
            throw new RefactoringException("");
        }

        if (QueryMethod.hasTheVariable(jmethod, newName)) {
            throw new RefactoringException("Not split:" +
              " the variable " + newName + " already exists in method " + jmethod.getName());
        }

        if (QueryClass.hasTheField(jclass, newName)) {
            throw new RefactoringException("Not split:" +
              " the field " + newName + " alreasy exists in class " + jclass.getName());
        }
    }

    protected void transform() throws RefactoringException {
        transformer = new SplitVariableTransformer(jv, cfg, pdg, nodesToBeChanged, newName);
        transformer.changeSrc();
        unparse(jfile);
    }

    protected String getLog() {
        return "into" + jvar.getName() + " and " + newName;
    }

    private GraphCompSet nodes = new GraphCompSet();

    private void collectNodesToBeChanged(PDGNode node) {
        if (node != null) {
            collectNodesToBeChanged(node, nodesToBeChanged);
        }
    }

    private CFGNode getCFGNode(JavaVariable var) {
        Element elem = (Element)var.getStatementNode();

        Iterator it = cfg.getNodes().iterator();
        while (it.hasNext()) {
            CFGNode n = (CFGNode)it.next();

            Element e = n.getElem();
            if (e != null && e.equals(elem)) {
                return n;
            }
        }
        return null;
    }

    private void collectNodesToBeChanged(PDGNode node, GraphCompSet prevnodes) {
        nodes.clear();
        traverseBackwardDD(node);

        GraphCompSet tmpNodes = new GraphCompSet(nodes);
        Iterator it = tmpNodes.iterator();
        while (it.hasNext()) {
            PDGNode n = (PDGNode)it.next();
            traverseForwardDD(n);
        }
        nodesToBeChanged = nodesToBeChanged.union(nodes);

        GraphCompSet diff = nodesToBeChanged.difference(prevnodes);
        it = diff.iterator();
        while (it.hasNext()) {
            PDGNode n = (PDGNode)it.next();
            collectNodesToBeChanged(n, nodesToBeChanged);
        }
    }

    private void traverseForwardDD(PDGNode node) {
        nodes.add(node);

        Iterator it = node.getOutgoingDDEdges().iterator();
        while (it.hasNext()) {
            DD edge = (DD)it.next();

            if (jvar.equals(edge.getVariable())) {
                PDGNode n = (PDGNode)edge.getDstNode();
                if (!nodes.contains(n)) {
                    traverseForwardDD(n);
                }
            }
        }
    }

    private void traverseBackwardDD(PDGNode node) {
        nodes.add(node);

        Iterator it = node.getIncomingDDEdges().iterator();
        while (it.hasNext()) {
            DD edge = (DD)it.next();

            if (jvar.equals(edge.getVariable())) {
                PDGNode n = (PDGNode)edge.getSrcNode();
                if (!nodes.contains(n)) {
                    traverseBackwardDD(n);
                }
            }
        }
    }

    private boolean possibleToSplit(GraphCompSet cnodes) {
        Iterator it = pdg.getNodes().difference(cnodes).iterator();
        while (it.hasNext()) {
            PDGNode node = (PDGNode)it.next();

            if (isDefinedAt(node.getCFGNode(), jv)) {
                return true;
            }
        }
        return false;
    }

    private boolean isDefinedAt(CFGNode node, JavaVariable jv) {
        if (node.hasDefVariable()) {
            CFGStatement n = (CFGStatement)node;
            if (n.containsDefVariable(jv)) {
                return true;
            }
        }
        return false;
    }
}
